/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2010 EADS - CASSIDIAN
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */

package org.ow2.weblab.portlet;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.ResourceBundle;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.EventRequest;
import javax.portlet.EventResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletRequestDispatcher;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.portlet.ResourceRequest;
import javax.portlet.ResourceResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.weblab.core.extended.factory.AnnotationFactory;
import org.ow2.weblab.core.extended.factory.ResourceFactory;
import org.ow2.weblab.core.extended.ontologies.WebLab;
import org.ow2.weblab.core.extended.util.ServiceUtil;
import org.ow2.weblab.core.helper.BeanHelper;
import org.ow2.weblab.core.helper.PoKHelper;
import org.ow2.weblab.core.helper.RDFHelperFactory;
import org.ow2.weblab.core.helper.impl.AdvancedSelector;
import org.ow2.weblab.core.helper.impl.RDFSelectorFactory;
import org.ow2.weblab.core.helper.impl.Statements;
import org.ow2.weblab.core.model.Annotation;
import org.ow2.weblab.core.model.PieceOfKnowledge;
import org.ow2.weblab.core.model.ResultSet;
import org.ow2.weblab.core.model.StringQuery;
import org.ow2.weblab.core.services.Analyser;
import org.ow2.weblab.core.services.Searcher;
import org.ow2.weblab.core.services.analyser.ProcessArgs;
import org.ow2.weblab.core.services.analyser.ProcessReturn;
import org.ow2.weblab.core.services.searcher.SearchArgs;
import org.ow2.weblab.core.services.searcher.SearchReturn;
import org.ow2.weblab.portlet.tool.AdvancedSearchConfigBean;
import org.ow2.weblab.portlet.tool.AdvancedSearchFieldBean;
import org.ow2.weblab.portlet.tool.DateFieldBean;
import org.ow2.weblab.portlet.tool.SearchServiceBean;
import org.ow2.weblab.portlet.tool.TextFieldBean;



/**
 * Portlet used to generate text requests. When request is submitted, a RequestEvent (request and service definition) is
 * sent. Portlet allow modification by user of the searcher service URL to call.
 * 
 * @author emilien, nbureau
 * 
 */
public class AdvancedSearchPortlet extends WebLabPortlet {
	private static Log logger = LogFactory.getLog(AdvancedSearchPortlet.class);

	protected static final String DEFAULT_SYNTAX_NAME = "lucene";
	private static final String DEFAULT_SEARCH = "def_mess";
	private static final String NEXT_DOCUMENTS_REACTION = "sendNextDocuments";
	private static final String FACET_SELECTION_REACTION = "facetSelection";
	private static final String FACET_UNSELECTION_REACTION = "facetUnselection";
	private static final String NEXT_DOCUMENTS_ACTION = "nextDocuments";
	
	public static String TEXT_KEYWORDS_INPUT = "textInput";
	public static String LAST_TEXT_KEYWORDS_INPUT = "lastTextInput";
	public static String QUERY_INPUT = "queryInput";
	public static String SUBMIT_INPUT = "submitInput";
	public static String USER_LAST_SEARCH = "user_last_search";
	public static String USER_ESB_CONF = "user_esb_conf";
	public static String SEARCH_ERROR = "searcher_error";
	public static String ESB_CONFIG_NAME = "esb_config_bean";
	public static String SEARCH_SERVICES_MAP = "search_services_map";
	public static String DISPLAY_CHECKING_POPUP = "display_checking_popup";
	public static String SEARCH_SERVICE = "searchServiceInput";
	public static String LAST_SEARCH_SERVICE = "last_search_service";
	public static String NATIVE_SYNTAX = "native_syntax";
	public static String NATIVE_SYNTAX_DISPLAYED = "native_syntax_displayed";

	private static String OFFSET_PROPERTY = WebLab.HAS_EXPECTED_OFFSET;
	private static String LIMIT_PROPERTY = WebLab.HAS_EXPECTED_LIMIT;
	private static String HTML_CONTENT_TYPE = "text/html; charset=utf-8";
	private static String TEXT_CONTENT_TYPE = "text/plain; charset=utf-8";
	
	protected static String SEARCH_ACTION = "search";
	protected static URL SEARCHER_WSDL_LOCATION;
	protected static Map<String, Searcher> searchers;
	protected static Map<String, Analyser> translators;
	protected static Map<String, String> usageContexts;
	protected AdvancedSearchConfigBean advanced_search_conf;
	protected static int distinct_submit_query;
	protected static boolean USE_REDIRECTION;

	private static final SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd");
	private static final Date OLDEST_DATE = new Date(0);


	
	@Override
	public void init() throws PortletException {
		/*
		 * init WebLab portlet for event mapping
		 */
		super.init();

		this.advanced_search_conf = BeanHelper.getInstance().getSpecificInstance(
				AdvancedSearchConfigBean.DEFAULT_CONF_FILE, true).getBean(AdvancedSearchConfigBean.DEFAULT_BEAN_NAME,
				AdvancedSearchConfigBean.class);
		/*
		 * initialize default configuration from portlet.xml
		 */

		try {
			/*
			 * getting searcher and analyser WSDL
			 */
			try {
				SEARCHER_WSDL_LOCATION = new File(getPortletContext().getRealPath(
						"WEB-INF/classes/services/WebLab.wsdl")).toURI().toURL();
			}
			catch (MalformedURLException e) {
				logger.error(e);
			}

			/*
			 * getting searchers and translators defined in AdvancedSearchConfigBean
			 */
			searchers = new HashMap<String, Searcher>();
			translators = new HashMap<String, Analyser>();
			usageContexts = new HashMap<String, String>();
			for (String serviceId : this.advanced_search_conf.getSearchServices().keySet()) {
				SearchServiceBean serviceBean = this.advanced_search_conf.getSearchServices().get(serviceId);
				// Add searcher
				Searcher searcher = ServiceUtil.getSearcherService(SEARCHER_WSDL_LOCATION,
						new URL(serviceBean.getURI()));
				searchers.put(serviceId, searcher);
				// Add usageContext if exists
				if (serviceBean.getUsageContext() != null && !serviceBean.getUsageContext().isEmpty()) { 
					usageContexts.put(serviceId, serviceBean.getUsageContext());
					logger.info("add uc: "+serviceBean.getUsageContext()+" for searcher "+serviceId);
				}

				// Add translator if exists
				if (serviceBean.getTranslatorClass() != null && !serviceBean.getTranslatorClass().isEmpty()) { 
					Class<?> clazz;
					try {
						clazz = Class.forName(serviceBean.getTranslatorClass());
	
						if (Analyser.class.isAssignableFrom(clazz)) {
							Analyser queryAnalyser = Analyser.class.cast(clazz.newInstance());
							translators.put(serviceId, queryAnalyser);
						}
					}
					catch (ClassNotFoundException e) {
						logger.error("Unbale to load QueryTranslator class for: " + serviceId);
					}
					catch (InstantiationException e) {
						logger.error("Unbale to instantiate QueryTranslator class for: " + serviceId);
					}
					catch (IllegalAccessException e) {
						logger.error("Access exception when trying to load QueryTranslator class for: " + serviceId);
					}
				}
				
				
			}
		} catch (Exception e) {
			logger.error("Unbale to get service URI");
		}

		/*
		 * setting the redirect parameter
		 */
		if (getInitParameter("use_redirection") != null) {
			USE_REDIRECTION = Boolean.parseBoolean(getInitParameter("use_redirection"));
		}
		else {
			USE_REDIRECTION = false;
		}
		distinct_submit_query = 0;
	}

	@Override
	public void destroy() {
		super.destroy();
	}

	/**
	 * Render HTML corresponding to the view mode
	 */
	@Override
	public void doView(RenderRequest req, RenderResponse res) throws IOException, PortletException {

		if (req.getParameter(SEARCH_ERROR) != null) {
			/*
			 * dispatch to jsp
			 */
			req.setAttribute(SEARCH_ERROR, req.getParameter(SEARCH_ERROR));
			logger.info("error in searcher");
			PortletRequestDispatcher rd = getPortletContext().getRequestDispatcher(getInitParameter("error_page_url"));
			rd.include(req, res);
		}

		if (req.getPortletSession().getAttribute(LAST_TEXT_KEYWORDS_INPUT) == null) {
			req.setAttribute(LAST_TEXT_KEYWORDS_INPUT, ResourceBundle.getBundle("search_portlet", req.getLocale())
					.getString("search.defaultText"));
			req.setAttribute(DEFAULT_SEARCH, true);
		}
		else {
			req.setAttribute(LAST_TEXT_KEYWORDS_INPUT, req.getPortletSession().getAttribute(LAST_TEXT_KEYWORDS_INPUT));
			req.setAttribute(DEFAULT_SEARCH, false);
		}

		req.setAttribute(SEARCH_SERVICES_MAP, this.advanced_search_conf.getSearchServices());
		req.setAttribute(DISPLAY_CHECKING_POPUP, this.advanced_search_conf.isDisplayCheckingPopup());

		if (req.getPortletSession().getAttribute(LAST_SEARCH_SERVICE) == null) {
			if (this.advanced_search_conf.getSearchServices().keySet().toArray().length > 0) {
				req.setAttribute(LAST_SEARCH_SERVICE,
						this.advanced_search_conf.getSearchServices().keySet().toArray()[0]);
			}
		} else {
			req.setAttribute(LAST_SEARCH_SERVICE, req.getPortletSession().getAttribute(LAST_SEARCH_SERVICE));
		}

		if (req.getPortletSession().getAttribute(NATIVE_SYNTAX) == null) {
			req.setAttribute(NATIVE_SYNTAX, false);
		} else {
			req.setAttribute(NATIVE_SYNTAX, req.getPortletSession().getAttribute(NATIVE_SYNTAX));
		}

		req.setAttribute(NATIVE_SYNTAX_DISPLAYED, this.advanced_search_conf.isNativeSyntaxOptionDisplayed());

		/*
		 * dispatch to jsp
		 */
		PortletRequestDispatcher rd = getPortletContext().getRequestDispatcher(getInitParameter("search_page_url"));
		rd.include(req, res);

	}

	@Override
	public void doEdit(RenderRequest req, RenderResponse res) throws PortletException, IOException {
		/*
		 * dispatch to jsp
		 */
		PortletRequestDispatcher rd = getPortletContext().getRequestDispatcher(getInitParameter("edit_page_url"));
		rd.include(req, res);
	}

	@Override
	public void processAction(ActionRequest req, ActionResponse res) throws IOException, PortletException {
		/*
		 * get the form name
		 */
		String form_name = req.getParameter("form_name");

		/*
		 * mapping
		 */
		if (form_name != null) {
			if (form_name.equals("search_query")) {
				List<AdvancedSearchFieldBean> advancedSearchFieldList = AdvancedSearchPortlet
						.getFieldListFromParameters(req.getParameterMap());

				req.getPortletSession().setAttribute("AdvancedSearchFieldBeanList", advancedSearchFieldList);
				req.getPortletSession().setAttribute("AdvancedSearchTextSearch", req.getParameter(TEXT_KEYWORDS_INPUT));

				String query = AdvancedSearchPortlet.generateQuery(req.getParameter(TEXT_KEYWORDS_INPUT),
						advancedSearchFieldList);
				/*
				 * create query
				 */
				StringQuery q = ResourceFactory.createResource(getDefaultNamespace() + getPortletName(), "queryID_"
						+ distinct_submit_query++, StringQuery.class);

				q.setRequest(query);
				
				/*
				 * setting nativeSyntax
				 */
				boolean isExpressedInNativeSyntax = false;
				if (req.getParameter("searchBox_nativeSyntax") != null
						&& req.getParameter("searchBox_nativeSyntax").equals("on")) {
					isExpressedInNativeSyntax = true;
				}

				req.getPortletSession().setAttribute(NATIVE_SYNTAX, isExpressedInNativeSyntax);

				/*
				 * Add isExpressedWith property
				 */
				Annotation annot = AnnotationFactory.createAndLinkAnnotation(q);
				PoKHelper h = RDFHelperFactory.getPoKHelperExtended(annot);
				if (isExpressedInNativeSyntax) {
					h.createLitStat(q.getUri(), WebLab.IS_EXPRESSED_WITH, req.getParameter(SEARCH_SERVICE));
				} else {
					h.createLitStat(q.getUri(), WebLab.IS_EXPRESSED_WITH, DEFAULT_SYNTAX_NAME);
				}
				
				logger.info("Query sent to index : " + q.getRequest());

				/*
				 * create args for search service
				 */
				SearchArgs args = new SearchArgs();
				args.setOffset(0);
				args.setLimit(10);
				args.setQuery(q);
				

				/*
				 * setting text query in user session
				 */
				req.getPortletSession().setAttribute(LAST_TEXT_KEYWORDS_INPUT, req.getParameter(TEXT_KEYWORDS_INPUT));

				/*
				 * setting search args in user session
				 */
				req.getPortletSession().setAttribute(USER_LAST_SEARCH, args);

				/*
				 * setting service used in user session
				 */
				req.getPortletSession().setAttribute(LAST_SEARCH_SERVICE, req.getParameter(SEARCH_SERVICE));

				/*
				 * setting nativeSyntax
				 */
				if (req.getParameter("searchBox_nativeSyntax") != null
						&& req.getParameter("searchBox_nativeSyntax").equals("on"))
					req.getPortletSession().setAttribute(NATIVE_SYNTAX, true);
				else
					req.getPortletSession().setAttribute(NATIVE_SYNTAX, false);

				if (q.getRequest() != null && q.getRequest() != "") {

					/*
					 * Use searcher selected by the user
					 */
					Searcher searcherToUse = searchers.get(req.getParameter(SEARCH_SERVICE));

					logger.info(req.getParameter(SEARCH_SERVICE) + " searcher is used.");

					
					/*
					 * process query to the searcher
					 */
					SearchReturn searchResult;
					try {

						if (searcherToUse != null) {
							String usageContext= usageContexts.get(req.getParameter(SEARCH_SERVICE));
							if (usageContext!=null && !usageContext.isEmpty())
								{
								args.setUsageContext(usageContext);
								logger.info("Usage context: "+usageContext + " is used.");			
								}
							searchResult = searcherToUse.search(args);
									

							/*
							 * sending a event and maybe redirect page
							 */
							if (USE_REDIRECTION) {
								sendEventForActionAndRedirect(SEARCH_ACTION, searchResult.getResultSet(), res);
							}
							else {
								sendEventForAction(SEARCH_ACTION, searchResult.getResultSet(), res);
							}
						}
					}
					catch (Exception e) {
						logger.error(e);
						res.setRenderParameter(SEARCH_ERROR, e.getMessage());
					}
				}
			} else if (form_name.equals("reset")) {
				req.getPortletSession().removeAttribute(LAST_TEXT_KEYWORDS_INPUT);
				req.getPortletSession().removeAttribute("AdvancedSearchFieldBeanList");
				ResultSet emptyRS = ResourceFactory.createResource("AdvancedSearchPortlet_ResultSet", ""+System.currentTimeMillis(), ResultSet.class);
				sendEventForAction("resetSearch", emptyRS, res);
			}
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public void serveResource(ResourceRequest request, ResourceResponse response) throws PortletException, IOException {

		/*
		 * serving advanced search field
		 */
		if (request.getParameter("page_part").equals("advancedSearchField")) {
			@SuppressWarnings("unchecked")
			List<AdvancedSearchFieldBean> fieldList = (List<AdvancedSearchFieldBean>) request.getPortletSession()
					.getAttribute("AdvancedSearchFieldBeanList");

			if (fieldList == null) {
				fieldList = new LinkedList<AdvancedSearchFieldBean>();
			}

			/*
			 * setting advanced search field list
			 */
			request.setAttribute("advancedSearchStructure", fieldList);
			/*
			 * setting boolean conditions
			 */
			request.setAttribute("boolean_conditions", this.advanced_search_conf.getBooleanConditions());
			/*
			 * setting field list
			 */
			request.setAttribute("advanced_search_fields", this.advanced_search_conf.getAdvancedSearchFields());
			/*
			 * dispatch to jsp
			 */
			response.setContentType(HTML_CONTENT_TYPE);
			PortletRequestDispatcher rd = getPortletContext().getRequestDispatcher(
					getInitParameter("advanced_search_field_page_url"));
			rd.include(request, response);

		}
		else if (request.getParameter("page_part").equals("nativeSyntax")) {
			Analyser translatorToUse = translators.get(request.getParameter("searchServiceInput"));

			logger.info(request.getParameter("searchServiceInput") + " translator is used.");

			List<AdvancedSearchFieldBean> fieldList = AdvancedSearchPortlet.getFieldListFromParameters(request
					.getParameterMap());
			String query = AdvancedSearchPortlet.generateQuery(request.getParameter(TEXT_KEYWORDS_INPUT), fieldList);

			/*
			 * create args for query translator
			 */
			StringQuery q = ResourceFactory.createResource(getDefaultNamespace() + getPortletName(), "queryID_"
					+ distinct_submit_query++, StringQuery.class);
			q.setRequest(query);

			ProcessArgs translatorArgs = new ProcessArgs();
			translatorArgs.setResource(q);
			ProcessReturn translatorResult;
			if (translatorToUse != null) {
				try {
					translatorResult = translatorToUse.process(translatorArgs);
					StringQuery resultQuery = (StringQuery) translatorResult.getResource();
					logger.info("query translated : " + resultQuery.getRequest());
					response.setContentType(TEXT_CONTENT_TYPE);
					response.getWriter().print(resultQuery.getRequest());
				}
				catch (Exception pe) {
					logger.error(pe);
					response.setProperty(SEARCH_ERROR, pe.getMessage());
				}
			}
			else {
				response.setContentType(TEXT_CONTENT_TYPE);
				response.getWriter().print(query);
			}
			response.getWriter().flush();
		}
		else if (request.getParameter("page_part").equals("advancedSearchNewLine")) {
			AdvancedSearchFieldBean newLine;

			boolean launchFormAfter = false;
			if (request.getParameter("launchFormAfter") != null)
				launchFormAfter = Boolean.parseBoolean(request.getParameter("launchFormAfter"));

			int position = 0;
			if (request.getParameter("position") != null)
				position = Integer.parseInt(request.getParameter("position"));

			if (request.getParameter("booleanCondition") != null && request.getParameter("fieldName") != null
					&& request.getParameter("fieldValue") != null && request.getParameter("fieldListValue") != null
					&& request.getParameter("fieldBeforeValue") != null
					&& request.getParameter("fieldAfterValue") != null) {
				String booleanCondition = request.getParameter("booleanCondition");

				// FieldName is composed by field type and name, try to retrieve the name :
				String[] fieldTypeAndName = request.getParameter("fieldName").split(":");
				String fieldType = fieldTypeAndName[0];
				String fieldName = fieldTypeAndName[1];

				if (fieldType.equals("date")) {
					String beforeValue = "";
					if (request.getParameter("fieldBeforeValue") != null
							&& !request.getParameter("fieldBeforeValue").isEmpty()) {
						beforeValue = request.getParameter("fieldBeforeValue");
					}
					String afterValue = "";
					if (request.getParameter("fieldAfterValue") != null
							&& !request.getParameter("fieldAfterValue").isEmpty())
						afterValue = request.getParameter("fieldAfterValue");

					newLine = new DateFieldBean(position, booleanCondition, fieldName, beforeValue, afterValue);
				} else if (fieldType.equals("list")) {
					String value = "";
					if (request.getParameter("fieldListValue") != null
							&& !request.getParameter("fieldListValue").isEmpty()) {
						value = request.getParameter("fieldListValue");
					}
					newLine = new TextFieldBean(position, AdvancedSearchFieldBean.Types.LIST_TYPE, booleanCondition,
							fieldName, value);
				} else {
					String value = "";
					if (request.getParameter("fieldValue") != null && !request.getParameter("fieldValue").isEmpty()) {
						value = request.getParameter("fieldValue");
					}
					newLine = new TextFieldBean(position, AdvancedSearchFieldBean.Types.TEXT_TYPE, booleanCondition,
							fieldName, value);
				}

				logger.info("New field added : " + newLine);

				/*
				 * setting new field
				 */
				request.setAttribute("advancedSearchNewLine", newLine);
				
				if (request.getPortletSession().getAttribute("AdvancedSearchFieldBeanList") != null) {
					((List<AdvancedSearchFieldBean>) request.getPortletSession().getAttribute("AdvancedSearchFieldBeanList")).add(newLine);
				} else {
					List<AdvancedSearchFieldBean> fieldList = new LinkedList<AdvancedSearchFieldBean>();
					fieldList.add(newLine);
					request.getPortletSession().setAttribute("AdvancedSearchFieldBeanList", fieldList);
				}
			}
			/*
			 * setting boolean conditions
			 */
			request.setAttribute("boolean_conditions", this.advanced_search_conf.getBooleanConditions());
			/*
			 * setting field list
			 */
			request.setAttribute("advanced_search_fields", this.advanced_search_conf.getAdvancedSearchFields());
			request.setAttribute("launchFormAfter", launchFormAfter);
			
			/*
			 * dispatch to jsp
			 */
			response.setContentType(HTML_CONTENT_TYPE);
			PortletRequestDispatcher rd = getPortletContext().getRequestDispatcher(
					getInitParameter("advanced_search_new_field_page_url"));
			rd.include(request, response);
		} else if (request.getParameter("page_part").equals("advancedSearchRemoveLine")) {
			response.setContentType(HTML_CONTENT_TYPE);
			int position = Integer.parseInt(request.getParameter("position"));
			
			logger.info("Removing field at position :"+position);
			((List<AdvancedSearchFieldBean>) request.getPortletSession().getAttribute("AdvancedSearchFieldBeanList")).remove(position);		
		
			/*
			 * setting advanced search field list
			 */
			request.setAttribute("advancedSearchStructure", (List<AdvancedSearchFieldBean>) request.getPortletSession().getAttribute("AdvancedSearchFieldBeanList"));
			/*
			 * setting boolean conditions
			 */
			request.setAttribute("boolean_conditions", this.advanced_search_conf.getBooleanConditions());
			/*
			 * setting field list
			 */
			request.setAttribute("advanced_search_fields", this.advanced_search_conf.getAdvancedSearchFields());
			
			/*
			 * dispatch to jsp
			 */
			response.setContentType(HTML_CONTENT_TYPE);
			PortletRequestDispatcher rd = getPortletContext().getRequestDispatcher(
					getInitParameter("advanced_search_field_page_url"));
			rd.include(request, response);

			
		} else if (request.getParameter("page_part").equals("generateQuery")) {
			List<AdvancedSearchFieldBean> fieldList = AdvancedSearchPortlet.getFieldListFromParameters(request
					.getParameterMap());
			String query = AdvancedSearchPortlet.generateQuery(request.getParameter(TEXT_KEYWORDS_INPUT), fieldList);
			response.setContentType(TEXT_CONTENT_TYPE);
			response.getWriter().print(query);
			response.getWriter().flush();
		} else if (request.getParameter("page_part").equals("getSelectValues")) {
			response.setContentType(TEXT_CONTENT_TYPE);

			String id = request.getParameter("id");
			Map<String, String> valuesMap;
			if (id.equals("format")) {
				valuesMap = this.advanced_search_conf.getFormatValues();
			} else if (id.equals("language")) {
				valuesMap = this.advanced_search_conf.getLanguageValues();
			} else {
				valuesMap = new HashMap<String, String>();
			}

			StringBuilder responseSB = new StringBuilder();
			responseSB.append("[{\"optionValue\": \"\", \"optionDisplay\": \"-\"},");
			for (Entry<String, String> entry : valuesMap.entrySet()) {
				responseSB.append('{');
				responseSB.append("\"optionValue\": ");
				responseSB.append("\"").append(entry.getKey()).append("\"");
				responseSB.append(", \"optionDisplay\": ");
				responseSB.append("\"").append(
						ResourceBundle.getBundle("search_portlet", request.getLocale()).getString(entry.getValue()))
						.append("\"");
				responseSB.append("},");
			}
			responseSB.deleteCharAt(responseSB.length() - 1);
			responseSB.append(']');

			response.getWriter().print(responseSB.toString());
			response.getWriter().flush();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.portlet.GenericPortlet#processEvent(javax.portlet.EventRequest, javax.portlet.EventResponse)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void processEvent(EventRequest request, EventResponse response) throws PortletException, IOException {
		long start = System.currentTimeMillis();
		String reaction = getReaction(request.getEvent().getQName()).getLocalPart();
		if (reaction.equals(NEXT_DOCUMENTS_REACTION)) {

			/*
			 * checking if the portlet is the query producer
			 */

			if (request.getEvent().getValue() != null && request.getEvent().getValue() instanceof PieceOfKnowledge) {
				PieceOfKnowledge pok = (PieceOfKnowledge) request.getEvent().getValue();
				AdvancedSelector rdfSelector = RDFSelectorFactory.getSelector(true);
				Statements map = rdfSelector.searchFor(pok);

				String queryURI = null;
				queryURI = map.getTypedValue(pok.getUri(), WebLab.HAS_QUERY, String.class);

				logger.debug("Map of pok : " + map);
				logger.debug("QueryID : " + queryURI);

				SearchArgs userSearchArg = (SearchArgs) request.getPortletSession().getAttribute(USER_LAST_SEARCH);

				if (userSearchArg != null && queryURI != null && userSearchArg.getQuery().getUri().equals(queryURI)) {
					/*
					 * this portlet has generated the query, redo search new offset and limit
					 */
					userSearchArg.setOffset(Integer.parseInt((String) map.entrySet().iterator().next().getValue().getValue(
							OFFSET_PROPERTY)));
					userSearchArg.setLimit(Integer.parseInt((String) map.entrySet().iterator().next().getValue().getValue(
							LIMIT_PROPERTY)));

					Searcher searcherToUse = searchers.get(request.getPortletSession()
							.getAttribute(LAST_SEARCH_SERVICE));

					logger.info(request.getPortletSession().getAttribute(LAST_SEARCH_SERVICE)
							+ " searcher is used : updating a resultSet, offset:" + userSearchArg.getOffset()
							+ " limit:" + userSearchArg.getLimit());
					/*
					 * process query to the searcher
					 */
					SearchReturn searchResult;
					try {
						if (searcherToUse != null) {
							searchResult = searcherToUse.search(userSearchArg);
							logger.info(searchResult.getResultSet());
							sendEventForAction(NEXT_DOCUMENTS_ACTION, searchResult.getResultSet(), response);
						}
					}
					catch (Exception e) {
						logger.error(e);
						response.setRenderParameter(SEARCH_ERROR, e.getMessage());
					}
				}
			}
		} else if (reaction.equals(FACET_SELECTION_REACTION) || reaction.equals(FACET_UNSELECTION_REACTION)) {
						
		
			if (request.getEvent().getValue() != null && request.getEvent().getValue() instanceof PieceOfKnowledge) {
				PieceOfKnowledge pok = (PieceOfKnowledge) request.getEvent().getValue();
				final AdvancedSelector rdfSelector = RDFSelectorFactory.getSelector(true);
				final Statements map = rdfSelector.searchFor(pok);

				final String queryURI = map.getTypedValue(pok.getUri(), WebLab.HAS_QUERY, String.class);

				logger.debug("Map of pok : " + map);
				logger.debug("QueryID : " + queryURI);

				final SearchArgs userSearchArg = (SearchArgs) request.getPortletSession().getAttribute(USER_LAST_SEARCH);
				logger.debug("User queryID: " + userSearchArg.getQuery().getUri());

				if (queryURI != null && userSearchArg.getQuery().getUri().equals(queryURI)) {
					/*
					 * this portlet has generated the query, redo search with filter
					 */
					if (userSearchArg.getQuery() instanceof StringQuery) {
						String filterValue = map.getTypedValue(pok.getUri(), getInitParameter("facetFilterQueryProperty"), String.class);
						StringQuery lastSearchQuery = (StringQuery) userSearchArg.getQuery();
						String lastSearch = lastSearchQuery.getRequest();

						String booleanOperator = filterValue.contains("AND") ? "AND" : "NOT";
						String fieldName = filterValue.replaceFirst(booleanOperator, "").split(":")[0].trim();
						String fieldValue = filterValue.replaceFirst(booleanOperator, "").split(":")[1].trim();

						List<AdvancedSearchFieldBean> fieldList;
						if (request.getPortletSession().getAttribute("AdvancedSearchFieldBeanList") != null) {
							fieldList = (List<AdvancedSearchFieldBean>) request.getPortletSession().getAttribute("AdvancedSearchFieldBeanList");
						} else {
							fieldList = new LinkedList<AdvancedSearchFieldBean>();
						}
						
						TextFieldBean facetFilterField = new TextFieldBean(-1, AdvancedSearchFieldBean.Types.TEXT_TYPE, booleanOperator, fieldName, fieldValue);
						
						if (reaction.equals(FACET_SELECTION_REACTION)) {
							lastSearchQuery.setRequest("(" + lastSearch + ") " + filterValue);
							fieldList.add(facetFilterField);
						} else if (reaction.equals(FACET_UNSELECTION_REACTION)) {
							lastSearchQuery.setRequest(lastSearch.replaceFirst("\\(", "").replaceFirst("\\)\\s*" + Pattern.quote(filterValue) + "($|\\s*\\))", "$1"));

							Iterator<AdvancedSearchFieldBean> fieldsIterator = fieldList.iterator();
							while (fieldsIterator.hasNext()) {
								AdvancedSearchFieldBean field = fieldsIterator.next();
								if (field instanceof TextFieldBean && field.getBooleanCondition().equals(booleanOperator) && field.getFieldName().equals(fieldName)
										&& ((TextFieldBean) field).getValue().equals(fieldValue)) {
									fieldsIterator.remove();
								}
							}
						}

						request.getPortletSession().setAttribute("AdvancedSearchFieldBeanList", fieldList);
						userSearchArg.setOffset(0);
						userSearchArg.setLimit(10);
						userSearchArg.setQuery(lastSearchQuery);
						
						Searcher searcherToUse = searchers.get(request.getPortletSession().getAttribute(LAST_SEARCH_SERVICE));
						logger.info(request.getPortletSession().getAttribute(LAST_SEARCH_SERVICE) + " searcher is used, create a facet filter: " + lastSearchQuery.getRequest());
						/*
						 * process query to the searcher
						 */
						final SearchReturn searchResult;
						try {
							if (searcherToUse != null) {
								searchResult = searcherToUse.search(userSearchArg);
								sendEventForAction(SEARCH_ACTION, searchResult.getResultSet(), response);
							}
						} catch (final Exception e) {
							logger.error(e);
							response.setRenderParameter(SEARCH_ERROR, e.getMessage());
						}
					} else {
						logger.error("Could not filter on non StringQuery requests.");
					}
				}
			}
		}
		logger.info("processEvent response time :" + Long.toString(System.currentTimeMillis() - start));
	}

	//privates void
	
	/**
	 * to get the search fields 
	 */
	protected static List<AdvancedSearchFieldBean> getFieldListFromParameters(Map<String, String[]> parameters) {
		List<AdvancedSearchFieldBean> advancedSearchFieldList = new LinkedList<AdvancedSearchFieldBean>();
		Pattern p = Pattern.compile("advanced_booleanCondition_(\\d+)");

		for (Entry<String, String[]> entry : parameters.entrySet()) {
			if (entry.getKey().startsWith("advanced_booleanCondition_")) {
				Matcher m = p.matcher(entry.getKey());
				m.matches();
				int position = Integer.parseInt(m.group(1));
				
				if (parameters.get("advanced_fieldName_" + position) != null
						&& parameters.get("advanced_fieldName_" + position)[0] != null
						&& !parameters.get("advanced_fieldName_" + position)[0].isEmpty()) {
					String[] fieldTypeAndName = parameters.get("advanced_fieldName_" + position)[0].split(":");
					String fieldType = fieldTypeAndName[0];
					String fieldName = fieldTypeAndName[1];

					boolean addField = true;
					AdvancedSearchFieldBean field;
					if (fieldType.equals("date")) {
						// By default, set before date to 01/01/1970
						String beforeValue = DATE_FORMAT.format(OLDEST_DATE);
						if (parameters.get("advanced_fieldBeforeValue_" + position) != null
								&& !parameters.get("advanced_fieldBeforeValue_" + position)[0].isEmpty()) {
							beforeValue = parameters.get("advanced_fieldBeforeValue_" + position)[0];
						}

						// By default, set after date to today
						String afterValue = DATE_FORMAT.format(Calendar.getInstance().getTime());
						if (parameters.get("advanced_fieldAfterValue_" + position) != null
								&& !parameters.get("advanced_fieldAfterValue_" + position)[0].isEmpty()) {
							afterValue = parameters.get("advanced_fieldAfterValue_" + position)[0];
						}

						field = new DateFieldBean(position, entry.getValue()[0], fieldName, beforeValue, afterValue);
					} else if (fieldType.equals("list")) {
						String value = "";
						if (parameters.get("advanced_fieldListValue_" + position) != null
								&& !parameters.get("advanced_fieldListValue_" + position)[0].isEmpty()) {
							value = parameters.get("advanced_fieldListValue_" + position)[0];
						} else {
							addField = false;
						}
						
						field = new TextFieldBean(position, AdvancedSearchFieldBean.Types.LIST_TYPE,
								entry.getValue()[0], fieldName, value);
					} else {
						String value = "";
						if (parameters.get("advanced_fieldValue_" + position) != null
								&& !parameters.get("advanced_fieldValue_" + position)[0].isEmpty()) {
							value = parameters.get("advanced_fieldValue_" + position)[0];
						} else {
							addField = false;
						}
						field = new TextFieldBean(position, AdvancedSearchFieldBean.Types.TEXT_TYPE,
								entry.getValue()[0], fieldName, value);
					}

					if (addField) {
						advancedSearchFieldList.add(field);
					}
				} else {
					logger.error("Name field was null for element " + position);
				}
			}
		}
		Collections.sort(advancedSearchFieldList);
		return advancedSearchFieldList;
	}

	/**
	 * To generate the query from a search field list
	 * @param textQuery
	 * @param advancedSearchFieldList
	 * @return
	 */
	protected static String generateQuery(String textQuery, List<AdvancedSearchFieldBean> advancedSearchFieldList) {
		// If text query is empty, remove the first boolean operator to make correct syntax
		if (textQuery.isEmpty() && advancedSearchFieldList.size() > 0)
			advancedSearchFieldList.get(0).setBooleanCondition("");

		StringBuilder generatedQuery = new StringBuilder();
		for (AdvancedSearchFieldBean advancedSearchFieldBean : advancedSearchFieldList) {
			if (advancedSearchFieldBean.getBooleanCondition() != null
					&& !advancedSearchFieldBean.getBooleanCondition().isEmpty())
				generatedQuery.append(advancedSearchFieldBean.getBooleanCondition()).append(" ");

			generatedQuery.append(advancedSearchFieldBean.getFieldName()).append(":");
			if (advancedSearchFieldBean instanceof TextFieldBean) {
				generatedQuery.append('(');
				generatedQuery.append(((TextFieldBean) advancedSearchFieldBean).getValue());
				generatedQuery.append(')');
			}
			else if (advancedSearchFieldBean instanceof DateFieldBean) {
				generatedQuery.append('[');
				generatedQuery.append(((DateFieldBean) advancedSearchFieldBean).getBeforeValue());
				generatedQuery.append(' ');
				generatedQuery.append(((DateFieldBean) advancedSearchFieldBean).getAfterValue());
				generatedQuery.append(']');
			}
			generatedQuery.append(' ');
		}
		if (generatedQuery.length() > 0)
			generatedQuery.deleteCharAt(generatedQuery.length() - 1);

		// return query
		if (generatedQuery.length() == 0)
			return textQuery;
		else if (textQuery.length() == 0)
			return generatedQuery.toString();
		else
			return '(' + textQuery + ')' + ' ' + generatedQuery.toString();
	}
}
